package org.openxma.dsl.platform.server;

import java.io.IOException;

import org.mortbay.jetty.Connector;
import org.mortbay.jetty.Handler;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.bio.SocketConnector;
import org.mortbay.jetty.webapp.WebAppContext;
import org.mortbay.jetty.webapp.WebInfConfiguration;
import org.mortbay.jetty.webapp.WebXmlConfiguration;
import org.mortbay.resource.Resource;

public class JettyApplication {
    private String webapp;
    private String contextPath;
    
    private static final Server SERVER = new Server();
    private static final String USAGE = 
            "Local Jetty Application for Fast-Development. "+
            "USAGE:\njava JettyApplication CONTEXT_PATH [WEBAPP_DIR]"+
            "Example:\njava JettyApplication /myapproot src/main/webapp"+
            "Parameters: \n"+
            "1) context path"+
            "2) webapp directory (optional, relative to the project root, default: src/main/webapp)";

    public static void main(String[] args) {
        JettyApplication app = new JettyApplication();
        System.out.println("Configure local jetty ...");
        if (args.length < 1) {
            System.err.println("Missing context path!\n");
            System.out.println(USAGE);
            System.exit(1);
        }
        app.webapp = "./src/main/webapp";
        if (args.length > 1) {
            app.webapp = "./" + args[1];
        }
        app.contextPath = args[0].startsWith("/") ? args[0] : "/" + args[0];
        app.start();
    }
    
    private void debugClassloader(ClassLoader loader) {
        System.out.println("Classloader " + loader);
        
        ClassLoader parent = loader.getParent();
        if (parent != null)
            debugClassloader(parent);
        
    }
    
    private void start() {
//        debugClassloader(this.getClass().getClassLoader());
        
        System.out.println("Starting local jetty application for Fast-Development ...");
        System.out.println("- contextPath: " + contextPath);
        System.out.println("- webapp: " + webapp);
        WebAppContext app = new WebAppContext();
        app.setContextPath(contextPath);
        app.setWar(webapp);
        String webDescriptor = System.getProperty("webXml");
        System.out.println("webxml path:" + webDescriptor);
        if (webDescriptor != null && !"".equals(webDescriptor.trim())) {
        	app.setDescriptor(webDescriptor);
        }
        
        
        try {
            Resource webInf = app.getWebInf();
            System.out.println("webInf absolutepath: "+webInf.getFile().getAbsolutePath());
            System.out.println("webInf is directory: "+webInf.isDirectory());
        } catch (IOException e1) {
            // TODO Auto-generated catch block
            System.out.println("Unable to read web inf directory");
        }
        
        // Avoid the taglib configuration because its a PITA if you don't have a
        // net connection
        app.setConfigurationClasses(new String[] {
                WebInfConfiguration.class.getName(),
                WebXmlConfiguration.class.getName() });
        app.setParentLoaderPriority(true);

        // We explicitly use the SocketConnector because the
        // SelectChannelConnector locks files
        Connector connector = new SocketConnector();
        connector.setPort(Integer.parseInt(System.getProperty("jetty.port",
                "8080")));
        connector.setMaxIdleTime(60000);

        JettyApplication.SERVER.setConnectors(new Connector[] { connector });
        JettyApplication.SERVER.setHandlers(new Handler[] { app });
        JettyApplication.SERVER.setAttribute(
                "org.mortbay.jetty.Request.maxFormContentSize", 0);
        JettyApplication.SERVER.setStopAtShutdown(true);

        try {
            JettyApplication.SERVER.start();
            System.out.println("Started local jetty ...");
            JettyApplication.SERVER.join();
            System.out.println("Ended local jetty ...");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
